@@ -30,6 +30,8 @@ class Instrument:
 					self.end = self.start + parameters["step"]
 		if 'end' in parameters:
 			self.end= self.params["end"]
+			if 'start' not in parameters:
+				self.start = 0
 
 		if 'step' in parameters:
 			self.step = parameters["step"]
@@ -66,7 +68,19 @@ class Instrument:
 		self.last_tick = (state["iterations"], state["tick"])
 
 
-	def play(self, state):
+	def get_note_length(self, state):
+		if 'let_ring' not in self.params:
+			return 1
+	
+		if not self.params['let_ring']:
+			return 1
+		
+		if 'note_length' in self.params:
+			return params["note_length"]
+
+		return state["ticks"] / state["meter"][1] 
+
+	def set_instrument(self):
 		if not(self.midi_set) and 'midi_instr' in self.params:
 			if not self.no_fluidsynth:
 				fluidsynth.set_instrument(self.params["channel"], self.params["midi_instr"])
@@ -75,22 +89,16 @@ class Instrument:
 			i.instrument_nr = self.params["midi_instr"]
 			self.track.instrument = i
 
+	def play(self, state):
+		if not(self.midi_set):
+			self.set_instrument()
+
 		if state["tick"] == 0:
-			if hasattr(self, "bar"):
-				self.track + self.bar
-			b = Bar()
-			b.set_meter(state["meter"])
-			b.length = 0.0
-			b.key = state["key"]
-			self.bar = b
-		let_ring = False
+			self.record_new_bar(state)
+
 		self.stop_playing_notes()
-		#if 'let_ring' not in self.params:
-		#else:
-		#	if not self.params['let_ring']:
-		#		self.stop_playing_notes()
-		#	else:
-		#		let_ring = True
+
+		note_length = self.get_note_length(state)
 
 		dur = state["resolution"]
 		if state["swing"]:
@@ -100,40 +108,57 @@ class Instrument:
 				dur *= (4.0 / 3.0)
 
 		n = self.generate_note(state)
-		v = self.generate_velocity(state)
+		v = int(self.generate_velocity(state))
+		c = self.params["channel"]
 		if n is not None and n != []:
 			for note in n:
-				note.velocity = int(v)
-				note.channel = self.params["channel"]
+				note.velocity = v
+				note.channel = c
 
-			# Record played notes in a Track
-			if self.last_tick != (state["iterations"], state["tick"]):
-				self.bar.place_notes(n, dur)
-				
-			else:
-				self.bar.bar[-1][2].add_notes(n)
+			self.record_notes(state, n, dur)
 
 			if not self.no_fluidsynth:
-				fluidsynth.play_NoteContainer(n,
-					self.params["channel"], v)
-			self.last_chan = self.params["channel"]
-			self.playing += n
+				fluidsynth.play_NoteContainer(n, c, v)
+
+			self.playing.append([n, note_length])
+			self.last_chan = c
 			self.last_played = n
 			if state["paint_function"] != None:
-				state["paint_function"](n, self.params["channel"])
+				state["paint_function"](n, c)
 		else:
-			if len(self.bar) > 0:
-				if self.bar[-1][1] is None:
-					i = 1 / (1.0 / dur + \
-						1.0 / self.bar[-1][1])
-					self.bar[-1][1] = i
-				else:
-					self.bar.place_notes(None, dur)
-			else:
-				self.bar.place_notes(None, dur)
+			self.record_rest(dur)
 
 		self.last_tick = (state["iterations"], state["tick"])
 
+	def record_new_bar(self, state):
+		if hasattr(self, "bar"):
+			self.track + self.bar
+		b = Bar()
+		b.set_meter(state["meter"])
+		b.length = 0.0
+		b.key = state["key"]
+		self.bar = b
+
+
+	def record_notes(self, state, notes, duration):
+		"""Adds notes to the track."""
+		if self.last_tick != (state["iterations"], state["tick"]):
+			self.bar.place_notes(notes, duration)	
+		else:
+			self.bar.bar[-1][2].add_notes(notes)
+
+	def record_rest(self, duration):
+		"""Adds a rest to the track, or adds a tick to the last\
+NoteContainer if there are notes playing."""
+		dur = duration
+		if len(self.bar) > 0:
+			if self.bar[-1][1] is None or len(self.playing) > 0:
+				i = 1 / (1.0 / dur + 1.0 / self.bar[-1][1])
+				self.bar[-1][1] = i
+			else:
+				self.bar.place_notes(None, dur)
+		else:
+			self.bar.place_notes(None, dur)
 
 
 	def generate_note(self, state):
@@ -158,6 +183,12 @@ class Instrument:
 
 
 	def stop_playing_notes(self):
-		if not self.no_fluidsynth:
-			fluidsynth.stop_NoteContainer(self.playing, self.last_chan)
-		self.playing = []
+		playing = []
+		for x in self.playing:
+			if x[1] <= 1:
+				if not self.no_fluidsynth:
+					fluidsynth.stop_NoteContainer(x[0], self.last_chan)
+			else:
+				playing.append([x[0], x[1] - 1])
+
+		self.playing = playing
