@@ -25,3 +25,20 @@ class Drum(Instrument):
 	def snare(self):
 		return [Note("E", 2)]
 
+	def lowest_tom(self):
+		return Note("F", 2)
+
+	def lower_tom(self):
+		return Note("G", 2)
+
+	def low_tom(self):
+		return Note("A", 2)
+
+	def middle_tom(self):
+		return Note("B", 2)
+
+	def high_tom(self):
+		return Note("C", 3)
+
+	def highest_tom(self):
+		return Note("D", 3)
